import PropTypes from 'prop-types';

import TooltipDisplay from './TooltipDisplay';
import TooltipTrigger from './TooltipTrigger';
import useTooltip from './_hooks/useTooltip';

/**
 * A simple tooltip that renders both the tooltip _trigger_ containing the
 * children passed to this component and the tooltip _display_ containing
 * whatever tooltip content is passed in. Tooltip content can be specified
 * two different ways:
 *
 * 1. by specifying the rendered content directly as the `content` prop, or
 * 2. by specifying a React component type as the `Component` prop and the
 *    props that should be passed to it when it's rendered as the `props` prop.
 *
 * If `content` is specified then it will take precedence.
 *
 * Note that a `TooltipTrigger` gets rendered around the tooltip's provided
 * children, which creates an extra DOM node to handle user interaction events.
 * By default this renders a plain `div`, which may create an unexpectedly
 * large hover target if you put a `Tooltip` (or a `TooltipTrigger` directly)
 * around, say, an `inline-block` component with a narrow width. If you want
 * a tooltip on a size-bound element, be sure to bound the size of the hover
 * target with a parent element.
 *
 * If you need more control over how and where the tooltip display container is
 * rendered, or if you need to share one display container across multiple
 * tooltip triggers, you can use the `useTooltip` hook to coordinate
 * `TooltipDisplay` and `TooltipTrigger` components that you render directly.
 * See the "advanced" stories for examples of how to do that.
 */
const Tooltip = function Tooltip(props) {
  const {
    content,
    props: childProps,
    children,
    isDesktop,
    breakpoint,
    ...triggerProps
  } = props;

  const tooltip = useTooltip({
    content,
    props: childProps,
    isDesktop,
    breakpoint,
  });

  return (
    <>
      <TooltipTrigger tooltip={tooltip} {...triggerProps}>
        {children}
      </TooltipTrigger>
      <TooltipDisplay tooltip={tooltip} />
    </>
  );
};

Tooltip.defaultProps = {
  content: undefined,
  Component: undefined,
  props: undefined,
  isDesktop: undefined,
  breakpoint: undefined,
};

Tooltip.propTypes = {
  /**
   * The content to render in this simple tooltip
   */
  content: PropTypes.node,
  /**
   * The component type to render the content of this tooltip
   */
  Component: PropTypes.elementType,
  /**
   * The props to pass to `Component` when rendering this tooltip
   */
  props: PropTypes.object, // eslint-disable-line react/forbid-prop-types
  /**
   * Flag indicating that screen size is "desktop" rather than "mobile". If
   * this is not provided then its value will be computed; if you already have
   * access to this information (say, in a Recoil store), pass it in to avoid
   * creating an additional resize listener.
   */
  isDesktop: PropTypes.bool,
  /**
   * The breakpoint at which to recompute `isDesktop`. Not used is `isDesktop`
   * is provided.
   */
  breakpoint: PropTypes.number,
  /**
   * The children to render as the tooltip trigger
   */
  children: PropTypes.node.isRequired,
};

export default Tooltip;
