import { useState, useEffect } from 'react';

/**
 * Hook to provide a computed fallback for whether we are at a mobile size or
 * a desktop size. In practice, many of our apps will already have this
 * information in their Recoil store, so if a `providedIsDesktop` value is
 * passed in here we don't bother recomputing the information we already have.
 */
const useIsDesktopFallback = function useIsDesktopFallback(opts = {}) {
  const {
    providedIsDesktop,
    breakpoint = 768,
  } = opts;

  // Computed value to use if we're not given a `providedIsDesktop` value
  const [isDesktop, setIsDesktop] = useState(false);

  // Effect to compute `isDesktop` value; does nothing if `providedIsDesktop`
  // is defined.
  useEffect(() => {
    if (typeof providedIsDesktop !== 'undefined') {
      return undefined;
    }

    const handleWindowResize = () => {
      setIsDesktop(window.innerWidth > breakpoint);
    };
    handleWindowResize();
    window.addEventListener('resize', handleWindowResize);
    return () => {
      window.removeEventListener('resize', handleWindowResize);
    };
  }, []);

  return (
    typeof providedIsDesktop === 'undefined' ? isDesktop : providedIsDesktop
  );
};

export default useIsDesktopFallback;
